@@ -22,7 +22,9 @@ class LabelDir(Enum):
 
 class RoleType(Enum):
     NC = "NC"
+    C_NAME = "C_NAME"
     IO = "IO"
+    IRQ = "IRQ"
     PHYSICAL = "PHYSICAL"
     PWR = "PWR"
     GND = "GND"
@@ -39,6 +41,8 @@ class RoleType(Enum):
     I2S = "I2S"
     SPI = "SPI"
     SD = "SD"
+    USB = "USB"
+    IRDA = "IRDA"
 
     PWM = "PWM"
     TMR = "TMR"
@@ -47,6 +51,7 @@ class RoleType(Enum):
 
     JTAG = "JTAG"
     SWD = "SWD"
+    FLASH = "FLASH"
 
 
 class IOType(Enum):
